#!/bin/sh

# To cater for Solaris
if test -d "/usr/xpg4/bin"; then
    PATH=/usr/xpg4/bin:$PATH
    export PATH
fi

TLA=tla
olddir=$(pwd)
srcdir=$(dirname $0)
cd "$srcdir"

emacs_is_beta=t
if type -p "$TLA" 1>/dev/null -a $TLA tree-version 1>/dev/null; then
	TREE_VERSION="$($TLA tree-version)"
	ARCH_VERSION="$($TLA logs -f|tail -n1)"
elif test -d "{arch}" -a -s "{arch}/++default-version"; then
	TREE_VERSION=$(cat "{arch}/++default-version")
	ARCH_VERSION="$TREE_VERSION--version-X"
else
	TREE_VERSION="--22.1.8"
	ARCH_VERSION="no_arch_version"
fi

emacs_major_version="$(echo $TREE_VERSION|sed -e s/"^.*--"//|cut -d . -f1)"
emacs_minor_version="$(echo $TREE_VERSION|sed -e s/"^.*--"//|cut -d . -f2)"
emacs_beta_version="$(echo $TREE_VERSION|sed -e s/"^.*--"//|cut -d . -f3)"
sxemacs_codename="Daimler"
sxemacs_arch_version="$ARCH_VERSION"

autoconf_ver=$(autoconf --version 2>/dev/null | head -n1)
autoheader_ver=$(autoheader --version 2>/dev/null | head -n1)
automake_ver=$(automake --version 2>/dev/null | head -n1)
aclocal_ver=$(aclocal --version 2>/dev/null | head -n1)
libtool_ver=$(libtool --version 2>/dev/null | head -n1)


# When things go wrong... get a bigger hammer!
_regexp='++log\|=b\(ui\)*ld'

if test -n "$PHAMMER"; then
    HAMMER=$PHAMMER
fi

if test -n "$HAMMER" -o -n "$REGEXP"; then
    if test -n "$REGEXP" -a "$HAMMER" != "BHFH"; then
        $TLA inventory -pbB|grep -v "$_regexp\|$REGEXP"|xargs rm -vrf
        unset REGEXP
    else
        if test "$HAMMER" = "BHFH"; then
            $TLA inventory -pjubB|xargs rm -vrf
        else
            $TLA inventory -pbB|grep -v "$_regexp"|xargs rm -vrf
        fi
    fi
    unset HAMMER
fi

cat>sxemacs_version.m4<<EOF
dnl autogenerated version number
m4_define([SXEM4CS_VERSION], [$emacs_major_version.$emacs_minor_version.$emacs_beta_version])
m4_define([SXEM4CS_MAJOR_VERSION], [$emacs_major_version])
m4_define([SXEM4CS_MINOR_VERSION], [$emacs_minor_version])
m4_define([SXEM4CS_BETA_VERSION], [$emacs_beta_version])
m4_define([SXEM4CS_BETA_P], [$emacs_is_beta])
m4_define([SXEM4CS_ARCH_VERSION], [$sxemacs_arch_version])
m4_define([SXEM4CS_CODENAME], [$sxemacs_codename])
m4_define([4UTOCONF_VERSION], [$autoconf_ver])
m4_define([4UTOHEADER_VERSION], [$autoheader_ver])
m4_define([4CLOCAL_VERSION], [$aclocal_ver])
m4_define([4UTOMAKE_VERSION], [$automake_ver])
m4_define([4IBTOOL_VERSION], [$libtool_ver])
EOF

if test -z "$FORCE"; then
    FORCE=
else
    rm -rf autom4te.cache aclocal.m4
    FORCE=--force
fi

if type glibtoolize 2>/dev/null; then
    LIBTOOLIZE=glibtoolize
else
    LIBTOOLIZE=libtoolize
fi

autoreconf $FORCE --verbose --install -Wall

# hack-o-matic.  Using gmp's config.{guess,sub} lets us have properer
# detected machine configurations --SY.
guess=$(grep GMP config.guess)
sub=$(grep GMP config.sub)
if test -z "${guess}"; then
    mv -vf config.guess configfsf.guess
    cp -v configgmp.guess config.guess
fi
if test -z "${sub}"; then
    mv -vf config.sub configfsf.sub
    cp -v configgmp.sub config.sub
fi

cd $olddir
